from datetime import datetime
import multiprocessing
import re
from threading import BoundedSemaphore
import time

import pytz
import tzlocal

from . import __version__
from .html_utils import *


class Scraper:
    """Provide web scraping functionality for www.punters.com.au"""

    AUSTRALIAN_STATES = (
        'australian-capital-territory',
        'new-south-wales',
        'northern-territory',
        'queensland',
        'south-australia',
        'tasmania',
        'victoria',
        'western-australia'
    )

    SOURCE_TIMEZONE = pytz.timezone('Australia/Melbourne')

    URL_ROOT = 'https://www.punters.com.au/'

    def __init__(self, http_client, html_parser, local_timezone=tzlocal.get_localzone(), concurrent_requests=multiprocessing.cpu_count() * 5):

        self.http_client = http_client
        self.parse_html = html_parser

        self.local_timezone = local_timezone

        self.request_lock = BoundedSemaphore(concurrent_requests)

    def fix_url(self, url, url_root=URL_ROOT):
        """Ensure the specified URL is fully qualified by prepending url_root if necessary"""

        if not re.search('[a-z]+://.*', url):
            if url.startswith('/') and url_root.endswith('/'):
                url = url_root[:-1] + url
            elif url.startswith('/') or url_root.endswith('/'):
                url = url_root + url
            else:
                url = url_root + '/' + url
        return url

    def get_html(self, url, retry_count=0, max_retries=5):
        """Get the root HTML element from the specified URL"""

        try:

            with self.request_lock:

                response = self.http_client.get(url)
                
                if response.status_code >= 500:
                    response.raise_for_status()
                elif response.status_code >= 400:
                    return None
                else:
                    return self.parse_html(response.text)

        except BaseException:

            if retry_count < max_retries:
                time.sleep(1)
                return self.get_html(url, retry_count + 1, max_retries)

            else:
                raise

    def is_compatible_with(self, version):
        """Return True if the current scraper version is compatible with the specified version"""

        return version.split('.')[0] == __version__.split('.')[0]
    
    def scrape_meets(self, date, meet_url_pattern='/({states})/'.format(states='|'.join(AUSTRALIAN_STATES))):
        """Scrape a list of meets occurring on the specified date"""

        meets = []

        try:
            date = self.local_timezone.localize(date)
        except ValueError:
            pass
        date = date.astimezone(self.SOURCE_TIMEZONE).replace(hour=0, minute=0, second=0, microsecond=0)
        date_string = date.strftime('%Y-%m-%d')

        html = self.get_html('https://www.punters.com.au/racing-results/{date}/'.format(date=date_string))
        if html is not None:

            for link in html.cssselect('a.label-link'):
                link_href = link.get('href')
                if date_string in link_href:
                    if meet_url_pattern is not None and re.search(meet_url_pattern, link_href):

                        meets.append({
                            'date':             date,
                            'track':            link.text_content().strip(),
                            'url':              self.fix_url(link.get('href')),
                            'scraper_version':  __version__
                        })

        return meets

    def scrape_races(self, meet):
        """Scrape a list of races occurring at the specified meet"""

        def parse_prize_pool(header_cell):
            groups = get_child_text_match_groups(header_cell, 'div.details-line span.capitalize', 'Total \$([\d.]+)([a-zA-Z])', index=0)
            if groups is not None and len(groups) > 0:
                prize_pool = float(groups[0])
                if len(groups) > 1:
                    if groups[1] in ('k', 'K'):
                        prize_pool *= 1000
                    elif groups[1] in ('m', 'M'):
                        prize_pool *= 1000000
                return prize_pool

        def parse_start_time(header_cell):
            value = parse_child_attribute(header_cell, 'div.details-line abbr.timestamp', 'data-utime', int)
            if value is not None:
                return datetime.fromtimestamp(value, self.SOURCE_TIMEZONE)

        races = []

        html = self.get_html(meet['url'])
        if html is not None:
            
            for table in html.cssselect('table.results-table'):
                header_cell = get_child(table, 'thead tr th')
                if header_cell is not None:

                    race = {
                        'number':           parse_child_text_match_group(header_cell, 'b.capitalize', '(\d+)', int),
                        'distance':         parse_child_attribute(header_cell, 'span.distance abbr.conversion', 'data-value', int),
                        'prize_pool':       parse_prize_pool(header_cell),
                        'track_condition':  get_child_text(header_cell, 'div.details-line span.capitalize', index=1),
                        'start_time':       parse_start_time(header_cell),
                        'url':              self.fix_url(get_child_attribute(header_cell, 'div.details-line span.capitalize a', 'href')),
                        'group':            None,
                        'entry_conditions': None,
                        'track_circ':       None,
                        'track_straight':   None,
                        'track_rail':       None,
                        'scraper_version':  __version__
                    }

                    html2 = self.get_html(race['url'])
                    if html2 is not None:

                        race['group'] = get_child_text(html2, 'span.event-name-title strong')

                        race['entry_conditions'] = [span.text_content().replace('.', '').strip() for span in html2.cssselect('div.event-details span.entry-conditions-text span')]

                        spans = html2.cssselect('div.event-details-bottom div span')
                        for index in range(0, len(spans) - 1, 2):
                            key = 'track_' + spans[index].text_content().strip().lower()

                            if key in ('track_circ', 'track_straight'):
                                race[key] = parse_child_attribute(spans[index + 1], 'abbr.conversion', 'data-value', int)

                            elif key == 'track_rail':
                                race[key] = spans[index + 1].text_content().strip()

                    races.append(race)

        return races

    def scrape_runners(self, race):
        """Scrape a list of runners competing in the specified race"""

        runners = []

        html = self.get_html(race['url'])
        if html is not None:
            
            for row in html.cssselect('table.form-guide-overview__table tbody tr'):

                runners.append({
                    'number':               parse_child_text_match_group(row, 'td.form-guide-overview__competitor-number', '(\d+)', int),
                    'is_scratched':         row.get('class') is not None and 'scratched' in row.get('class').lower(),
                    'horse_url':            self.fix_url(get_child_attribute(row, 'a.form-guide-overview__horse-link', 'href')),
                    'horse_has_blinkers':   get_child(row, 'div.has-blinkers') is not None,
                    'jockey_url':           self.fix_url(get_child_attribute(row, 'a.form-guide-overview__jockey-link', 'href')),
                    'jockey_is_apprentice': get_child(row, 'a.form-guide-overview__jockey-link span') is not None,
                    'jockey_claiming':      parse_child_text_match_group(row, 'a.form-guide-overview__jockey-link span', '\(a([\d.]+)\)', float, default=0.0),
                    'trainer_url':          self.fix_url(get_child_attribute(row, 'a.form-guide-overview__trainer-link', 'href')),
                    'weight':               parse_child_text(row, 'td.form-guide-overview__competitor-weight', float),
                    'barrier':              parse_child_text(row, 'td.form-guide-overview__competitor-barrier', int),
                    'scraper_version':      __version__
                })

        return runners

    def scrape_profile(self, url):
        """Scrape a profile from the specified URL"""

        def parse_name(html):
            name = get_child_text(html, 'h1')
            if name is not None:
                span_text = get_child_text(html, 'h1 span')
                if span_text is not None:
                    name = name.replace(span_text, '').strip()
            return name

        if url != self.URL_ROOT:
            html = self.get_html(url)
            if html is not None:

                profile = {
                    'url':              url,
                    'name':             parse_name(html),
                    'scraper_version':  __version__
                }

                for row in html.cssselect('div.moduleItem table tr'):
                    label = get_child_text(row, 'th')
                    if label is not None:
                        label = label.replace(':', '').lower()

                        if label == 'profile':
                            groups = get_child_text_match_groups(row, 'td', 'year old (.*) (.*)')
                            if groups is not None and len(groups) > 1:
                                profile['colour'] = groups[0]
                                profile['sex'] = groups[1]

                        elif label == 'pedigree':
                            groups = get_child_text_match_groups(row, 'td', '(.*) x (.*)')
                            if groups is not None and len(groups) > 1:
                                profile['sire'] = groups[0]
                                profile['dam'] = groups[1]

                        elif label == 'country':
                            profile['country'] = get_child_text(row, 'td')

                        elif label == 'foaled':
                            value = get_child_text(row, 'td')
                            if value is not None:
                                profile['foaled'] = self.SOURCE_TIMEZONE.localize(datetime.strptime(value, '%d/%m/%Y'))

                return profile

    def scrape_horse(self, runner):
        """Scrape the profile of the horse associated with the specified runner"""

        return self.scrape_profile(runner['horse_url'])

    def scrape_jockey(self, runner):
        """Scrape the profile of the jockey associated with the specified runner"""

        return self.scrape_profile(runner['jockey_url'])

    def scrape_trainer(self, runner):
        """Scrape the profile of the trainer associated with the specified runner"""

        return self.scrape_profile(runner['trainer_url'])

    def scrape_performances(self, profile):
        """Scrape a list of performances associated with the specified profile"""

        def parse_date(ul):
            value = get_child_text(ul, 'li.timeline-disc b span.date')
            if value is not None:
                return self.SOURCE_TIMEZONE.localize(datetime.strptime(value, '%d-%b-%y'))

        def parse_money(ul, pattern):
            value = get_child_text_match_group(ul, 'li.timeline-disc', pattern)
            if value is not None:
                value = value.replace(',', '')
                value = re.sub('\.+', '.', value)
                while value[0] == '.':
                    value = value[1:]
                while value[-1] == '.':
                    value = value[:-1]
                return float(value)

        def parse_winning_time(ul):
            value = get_child_text_match_group(ul, 'li.timeline-disc', 'Winning Time: ([\d:.]+)')
            if value is not None:
                value_parts = value.split(':')
                if len(value_parts) > 0:
                    winning_time = float(value_parts[-1])
                    if len(value_parts) > 1:
                        winning_time += int(value_parts[0]) * 60
                    return round(winning_time, 2)

        performances = []

        html = self.get_html(profile['url'])
        if html is not None:
            
            for ul in html.cssselect('ul.timeline'):
                if 'TRIAL' not in ul.text_content().upper():
                    result_text = get_child_text(ul, 'span.formSummaryPosition')
                    if result_text != 'Abn':

                        performance = {
                            'track':            get_child_text(ul, 'li.timeline-disc b span.simlight'),
                            'date':             parse_date(ul),
                            'distance':         parse_child_text_match_group(ul, 'li.timeline-disc span.dist', '(\d+)', int),
                            'track_condition':  get_child_text(ul, 'span.badge'),
                            'prize_money':      parse_money(ul, '\$([\d,.]+) \(of'),
                            'prize_pool':       parse_money(ul, '\(of \$([\d,.]+)'),
                            'barrier':          parse_child_text_match_group(ul, 'li.timeline-disc', 'Barrier (\d+)', int),
                            'winning_time':     parse_winning_time(ul),
                            'starting_price':   parse_money(ul, 'SP: \$([\d.]+)'),
                            'horse_url':        None,
                            'jockey_url':       None,
                            'weight':           None,
                            'carried':          None,
                            'lengths':          parse_child_text_match_group(ul, 'span.timeline-right.placed', '([\d.]+)L', float, default=0.0),
                            'result':           try_parse(result_text, int),
                            'starters':         parse_child_text(ul, 'span.starters', int),
                            'scraper_version':  __version__
                        }

                        for link in ul.cssselect('span.timeline-right.placed a'):
                            link_href = link.get('href')
                            for key in ('horse', 'jockey'):
                                if key in link_href:
                                    performance[key + '_url'] = self.fix_url(link_href)
                                    break

                        weights = [float(abbr.get('data-value')) for abbr in ul.cssselect('span.timeline-right.placed abbr') if abbr.get('data-type') == 'weight']
                        if len(weights) > 0:
                            performance['weight'] = performance['carried'] = weights[-1]
                            if len(weights) > 1:
                                performance['weight'] = weights[0]

                        performances.append(performance)

        return performances



import cache_requests
from lxml import html
import punters_client
import pytest
import pytz
import redis
import requests


@pytest.fixture(scope='session')
def scraper():

    http_client = None
    try:
        http_client = cache_requests.Session(connection=redis.fromurl('redis://localhost:6379/punters_client_test'))
    except BaseException:
        try:
            http_client = cache_requests.Session()
        except BaseException:
            http_client = requests.Session()

    html_parser = html.fromstring

    return punters_client.Scraper(http_client, html_parser)


@pytest.fixture(scope='session')
def source_timezone():

    return pytz.timezone('Australia/Melbourne')


@pytest.fixture(scope='session')
def runner():

    return {
        'number':               1,
        'is_scratched':         False,
        'horse_url':            'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
        'horse_has_blinkers':   False,
        'jockey_url':           'https://www.punters.com.au/jockeys/Ben-E-Thompson_2143/',
        'jockey_is_apprentice': True,
        'jockey_claiming':      1.5,
        'trainer_url':          'https://www.punters.com.au/trainers/Scott-McIntosh_1010/',
        'weight':               61.0,
        'barrier':              2,
        'scraper_version':      punters_client.__version__
    }



import punters_client
import pytest


@pytest.fixture(scope='module')
def expected_values(runner, source_timezone):

    return {
        'url':              runner['jockey_url'],
        'name':             'Ben E Thompson',
        'scraper_version':  punters_client.__version__
    }


@pytest.fixture(scope='module')
def scraped_values(runner, scraper):

    return scraper.scrape_jockey(runner)


def test_expected_values(expected_values, scraped_values):
    """The scrape_jockey method should return a dictionary containing all expected values"""

    assert scraped_values == expected_values


def test_in_memorium(scraper):
    """The scrape_jockey method should return the expected values on jockey in memorium profiles"""

    runner = {
        'jockey_url':   'https://www.punters.com.au/jockeys/Timothy-Bell_1685/'
    }

    expected_values = {
        'url':              'https://www.punters.com.au/jockeys/Timothy-Bell_1685/',
        'name':             'Timothy Bell',
        'scraper_version':  punters_client.__version__
    }

    scraped_values = scraper.scrape_jockey(runner)

    assert scraped_values == expected_values



from datetime import datetime

import punters_client
import pytest

from common import check_expected_items, check_unexpected_items


@pytest.fixture(scope='module', params=[1, 2])
def meet_id(request):

    return request.param


@pytest.fixture(scope='module')
def meet(meet_id, source_timezone):

    if meet_id == 1:
        return {
            'date':             source_timezone.localize(datetime(2016, 2, 1)),
            'track':            'Kilmore',
            'url':              'https://www.punters.com.au/racing-results/victoria/Kilmore/2016-02-01/',
            'scraper_version':  punters_client.__version__
        }
    elif meet_id == 2:
        return {
            'date':             source_timezone.localize(datetime(2015, 11, 3)),
            'track':            'Flemington',
            'url':              'https://www.punters.com.au/racing-results/victoria/Flemington/2015-11-03/',
            'scraper_version':  punters_client.__version__
        }


@pytest.fixture(scope='module')
def expected_races(meet_id, source_timezone):

    if meet_id == 1:
        return [
            {
                'number':           1,
                'distance':         1100,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 13, 10)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Wandong-Bus-and-Coach-Maiden-Plate_411192/',
                'group':            '',
                'entry_conditions': [
                    'Three-Years-Old and Upwards',
                    'Maiden',
                    'No sex restriction',
                    'Set Weights'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           2,
                'distance':         1200,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 13, 45)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Book-now-for-Valentines-Day-14-Feb-Maiden-Plate_411193/',
                'group':            '',
                'entry_conditions': [
                    'Three-Years-Old',
                    'Maiden',
                    'Fillies',
                    'Set Weights'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           3,
                'distance':         1450,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 14, 20)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Montmorency-RSL-Maiden-Plate_411194/',
                'group':            '',
                'entry_conditions': [
                    'Three-Years-Old and Upwards',
                    'Maiden',
                    'No sex restriction',
                    'Set Weights'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           4,
                'distance':         1600,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 14, 55)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Sunday-Sessions-%40-Trackside-7-Feb-Maiden-Plate_411195/',
                'group':            '',
                'entry_conditions': [
                    'Three-Years-Old and Upwards',
                    'Maiden',
                    'No sex restriction',
                    'Set Weights'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           5,
                'distance':         1100,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 15, 32)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/www.kilmoreracingclub.com.au-BM64-Handicap_411196/',
                'group':            '',
                'entry_conditions': [
                    'No age restriction',
                    'BenchMark 64',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           6,
                'distance':         1200,
                'prize_pool':       16000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 16, 12)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Wilson-Medic-One-Rising-Stars-0-58-Handicap_411197/',
                'group':            '',
                'entry_conditions': [
                    'No age restriction',
                    '0 - 58',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           7,
                'distance':         1900,
                'prize_pool':       20000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 16, 52)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Sportsmans-Lunch-%40-Trackside-26-Feb-BM64-Handicap_411198/',
                'group':            '',
                'entry_conditions': [
                    'No age restriction',
                    'BenchMark 64',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           8,
                'distance':         1600,
                'prize_pool':       16000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2016, 2, 1, 17, 32)),
                'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/Book-Now-Jet-Roofing-Kilmore-Pacing-Cup-0-58-Handicap_411199/',
                'group':            '',
                'entry_conditions': [
                    'No age restriction',
                    '0 - 58',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       1735,
                'track_straight':   320,
                'track_rail':       'True Entire Circuit',
                'scraper_version':  punters_client.__version__
            }
        ]
    elif meet_id == 2:
        return [
            {
                'number':           1,
                'distance':         1000,
                'prize_pool':       150000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 10, 40)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Emirates-Airline-Plate_382827/',
                'group':            'G3',
                'entry_conditions': [
                    'Two-Years-Old',
                    'No class restriction',
                    'Fillies',
                    'Set Weights'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           2,
                'distance':         1700,
                'prize_pool':       100000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 11, 20)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/tab.com.au-Trophy_382828/',
                'group':            '',
                'entry_conditions': [
                    'Four-Years-Old and Upwards',
                    'No class restriction',
                    'Mares',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           3,
                'distance':         2800,
                'prize_pool':       100000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 12, 00)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/J.B.-Cummings-AM-Tribute-Plate_382829/',
                'group':            '',
                'entry_conditions': [
                    'No age restriction',
                    'BenchMark 96',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           4,
                'distance':         1400,
                'prize_pool':       100000.00,
                'track_condition':  'Good 4',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 12, 40)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Lavazza-Short-Black_382830/',
                'group':            '',
                'entry_conditions': [
                    'Four-Years-Old and Five-Years-Old',
                    'BenchMark 90',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           5,
                'distance':         1000,
                'prize_pool':       100000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 13, 20)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Schweppes-%23FlemingtonFling_382831/',
                'group':            '',
                'entry_conditions': [
                    'Three-Years-Old',
                    'No class restriction',
                    'No sex restriction',
                    'Set Weights plus Penalties'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           6,
                'distance':         1400,
                'prize_pool':       150000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 14, 00)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Lexus-Hybrid-Plate_382832/',
                'group':            'LR',
                'entry_conditions': [
                    'Three-Years-Old',
                    'No class restriction',
                    'Fillies',
                    'Set Weights plus Penalties'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           7,
                'distance':         3200,
                'prize_pool':       6000000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 15, 00)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Emirates-Melbourne-Cup_383401/',
                'group':            'G1',
                'entry_conditions': [
                    'No age restriction',
                    'No class restriction',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           8,
                'distance':         1800,
                'prize_pool':       150000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 15, 55)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/James-Boag%E2%80%99s-Symphony-Stakes_382833/',
                'group':            'LR',
                'entry_conditions': [
                    'No age restriction',
                    'No class restriction',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           9,
                'distance':         1200,
                'prize_pool':       150000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 16, 35)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/MSS-Security-Sprint_382834/',
                'group':            'LR',
                'entry_conditions': [
                    'No age restriction',
                    'No class restriction',
                    'No sex restriction',
                    'Handicap'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            },
            {
                'number':           10,
                'distance':         1400,
                'prize_pool':       200000.00,
                'track_condition':  'Good 3',
                'start_time':       source_timezone.localize(datetime(2015, 11, 3, 17, 15)),
                'url':              'https://www.punters.com.au/form-guide/Flemington_64271/Hong-Kong-Jockey-Club-Stakes_382835/',
                'group':            'G3',
                'entry_conditions': [
                    'Four-Years-Old and Upwards',
                    'No class restriction',
                    'Mares',
                    'Set Weights plus Penalties'
                ],
                'track_circ':       2313,
                'track_straight':   448,
                'track_rail':       '-',
                'scraper_version':  punters_client.__version__
            }
        ]


@pytest.fixture(scope='module')
def scraped_races(meet, scraper):

    return scraper.scrape_races(meet)


def test_expected_races(expected_races, scraped_races):
    """The scrape_races method should return a list of dictionaries containing all expected values"""

    check_expected_items(expected_races, scraped_races, ['number'])


def test_unexpected_races(expected_races, scraped_races):
    """The scrape_races method should return a list of dictionaries that does not contain any unexpected values"""

    check_unexpected_items(expected_races, scraped_races, ['number'])



import punters_client
import pytest


@pytest.fixture(scope='module')
def expected_values(runner, source_timezone):

    return {
        'url':              runner['trainer_url'],
        'name':             'Scott McIntosh',
        'scraper_version':  punters_client.__version__
    }


@pytest.fixture(scope='module')
def scraped_values(runner, scraper):

    return scraper.scrape_trainer(runner)


def test_expected_values(expected_values, scraped_values):
    """The scrape_trainer method should return a dictionary containing all expected values"""

    assert scraped_values == expected_values



from datetime import datetime

import punters_client
import pytest
import tzlocal

from common import check_expected_items, check_unexpected_items


@pytest.fixture(scope='module', params=[datetime(2016, 2, 1), tzlocal.get_localzone().localize(datetime(2016, 2, 1))])
def date(request):

    return request.param


@pytest.fixture(scope='module')
def expected_meets(date, source_timezone):

    try:
        date = source_timezone.localize(date)
    except ValueError:
        date = date.astimezone(source_timezone)
    date = date.replace(hour=0, minute=0, second=0, microsecond=0)

    return [
        {
            'date':             date,
            'track':            'Kilmore',
            'url':              'https://www.punters.com.au/racing-results/victoria/Kilmore/2016-02-01/',
            'scraper_version':  punters_client.__version__
        },
        {
            'date':             date,
            'track':            'Nowra',
            'url':              'https://www.punters.com.au/racing-results/new-south-wales/Nowra/2016-02-01/',
            'scraper_version':  punters_client.__version__
        }
    ]


@pytest.fixture(scope='module')
def scraped_meets(date, scraper):

    return scraper.scrape_meets(date)


def test_expected_meets(expected_meets, scraped_meets):
    """The scrape_meets method should return a list of dictionaries containing all expected values"""

    check_expected_items(expected_meets, scraped_meets, ['date', 'track'])


def test_unexpected_meets(expected_meets, scraped_meets):
    """The scrape_meets method should return a list of dictionaries that does not contain any unexpected values"""

    check_unexpected_items(expected_meets, scraped_meets, ['date', 'track'])



import re


def get_child(parent, selector, index=0):
    """Get a child element from parent matching the specified CSS selector at the specified zero-based index"""

    children = parent.cssselect(selector)
    if children is not None and ((index >= 0 and len(children) > index) or (index < 0 and len(children) >= abs(index))):
        return children[index]


def get_child_attribute(parent, selector, attribute, index=0):
    """Get an attribute value from the specified child element"""

    child = get_child(parent, selector, index)
    if child is not None:
        return child.get(attribute)


def parse_child_attribute(parent, selector, attribute, parser, index=0, default=None):
    """Parse an attribute value from the specified child element"""

    return try_parse(get_child_attribute(parent, selector, attribute, index), parser, default)


def get_child_text(parent, selector, index=0):
    """Get the text content of the specified child element"""

    child = get_child(parent, selector, index)
    if child is not None:
        return child.text_content().strip()


def parse_child_text(parent, selector, parser, index=0, default=None):
    """Parse the text content of the specified child element"""

    return try_parse(get_child_text(parent, selector, index), parser, default)


def get_child_text_match(parent, selector, pattern, index=0):
    """Get a regex match on the specified child element's text content"""

    text = get_child_text(parent, selector, index)
    if text is not None:
        return re.search(pattern, text)


def get_child_text_match_groups(parent, selector, pattern, index=0):
    """Get the regex match groups for the specified child element's text content"""

    match = get_child_text_match(parent, selector, pattern, index)
    if match is not None:
        return match.groups()


def get_child_text_match_group(parent, selector, pattern, index=0, group=0):
    """Get a regex match group for the specified child element's text content"""

    groups = get_child_text_match_groups(parent, selector, pattern, index)
    if groups is not None and ((group >= 0 and len(groups) > group) or (group < 0 and len(groups) >= abs(group))):
        return groups[group]


def parse_child_text_match_group(parent, selector, pattern, parser, index=0, group=0, default=None):
    """Parse a regex match group for the specified child element's text content"""

    return try_parse(get_child_text_match_group(parent, selector, pattern, index, group), parser, default)


def try_parse(value, parser, default=None):
    """Try to parse value using parser, returning default if the parse fails"""

    try:
        return parser(value)
    except (TypeError, ValueError):
        return default


import re

import punters_client
import pytest


@pytest.fixture(scope='module')
def version_parts():

    return re.search('(\d+).(\d+).(\d+)([a-z]+)?(\d+)?', punters_client.__version__).groups()


def generate_version(parts):
    """Generate a version number string from the specified parts"""

    version = '.'.join([str(part) for part in parts[0:2]])
    if len(parts) > 3:
        version += parts[3]
        if len(parts) > 4:
            version += str(parts[4])
    return version


def test_equal(scraper):
    """is_compatible_with(same_version) should return True"""

    assert scraper.is_compatible_with(punters_client.__version__) is True


def test_major(scraper, version_parts):
    """is_compatible_with(new_major_version) should return False"""

    new_version_parts = list(version_parts)
    new_version_parts[0] = int(new_version_parts[0]) + 1

    assert scraper.is_compatible_with(generate_version(new_version_parts)) is False


def test_minor(scraper, version_parts):
    """is_compatible_with(new_minor_version) should return True"""

    new_version_parts = list(version_parts)
    new_version_parts[1] = int(new_version_parts[1]) + 1

    assert scraper.is_compatible_with(generate_version(new_version_parts)) is True


def test_patch(scraper, version_parts):
    """is_compatible_with(new_patch_version) should return True"""

    new_version_parts = list(version_parts)
    new_version_parts[2] = int(new_version_parts[2]) + 1

    assert scraper.is_compatible_with(generate_version(new_version_parts)) is True


def test_pre_release(scraper, version_parts):
    """is_compatible_with(new_pre_release_version) should return True"""

    new_version_parts = list(version_parts)
    if len(new_version_parts) > 4:
        new_version_parts[4] = int(new_version_parts[4]) + 1
    elif len(new_version_parts) > 3:
        new_version_parts.append(1)
    else:
        new_version_parts.extend(['a', 1])

    assert scraper.is_compatible_with(generate_version(new_version_parts)) is True



from datetime import datetime

import punters_client
import pytest

from common import check_expected_items, check_unexpected_items


@pytest.fixture(scope='module')
def race(source_timezone):

    return {
        'number':           5,
        'distance':         1100,
        'prize_pool':       20000.00,
        'track_condition':  'Good 4',
        'start_time':       source_timezone.localize(datetime(2016, 2, 1, 15, 32)),
        'url':              'https://www.punters.com.au/form-guide/Kilmore_69146/www.kilmoreracingclub.com.au-BM64-Handicap_411196/',
        'scraper_version':  punters_client.__version__
    }


@pytest.fixture(scope='module')
def expected_runners():

    return [
        {
            'number':               1,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Ben-E-Thompson_2143/',
            'jockey_is_apprentice': True,
            'jockey_claiming':      1.5,
            'trainer_url':          'https://www.punters.com.au/trainers/Scott-McIntosh_1010/',
            'weight':               61.0,
            'barrier':              2,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               2,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Chima_325128/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Cory-Parish_2656/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/David-Hayes-and-Tom-Dabernig_897/',
            'weight':               58.5,
            'barrier':              7,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               3,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Thisgoingsomewhere_214653/',
            'horse_has_blinkers':   True,
            'jockey_url':           'https://www.punters.com.au/jockeys/Jarrod-Fry_686/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/John-and-Chris-Ledger_938/',
            'weight':               58.0,
            'barrier':              5,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               4,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Azzcool_158971/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Chris-Symons_291/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/Alexander-Rae_13641/',
            'weight':               57.5,
            'barrier':              4,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               6,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Love-And-Lies_315909/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Rebecca-Williams_2263/',
            'jockey_is_apprentice': True,
            'jockey_claiming':      1.5,
            'trainer_url':          'https://www.punters.com.au/trainers/Jennifer-Williams_4610/',
            'weight':               57.0,
            'barrier':              10,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               7,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Our-Hussey_81386/',
            'horse_has_blinkers':   True,
            'jockey_url':           'https://www.punters.com.au/jockeys/Mitchell-Aitken_9402/',
            'jockey_is_apprentice': True,
            'jockey_claiming':      3.0,
            'trainer_url':          'https://www.punters.com.au/trainers/Russell-Green_8566/',
            'weight':               57.0,
            'barrier':              3,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               8,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Aglaia_316893/',
            'horse_has_blinkers':   True,
            'jockey_url':           'https://www.punters.com.au/jockeys/Brandon-Stockdale_8506/',
            'jockey_is_apprentice': True,
            'jockey_claiming':      1.5,
            'trainer_url':          'https://www.punters.com.au/trainers/Mark-Riley_841/',
            'weight':               56.5,
            'barrier':              9,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               10,
            'is_scratched':         False,
            'horse_url':            'https://www.punters.com.au/horses/Sullivan-Bay_322748/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Jye-McNeil_3183/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/David-Hayes-and-Tom-Dabernig_897/',
            'weight':               55.5,
            'barrier':              1,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               5,
            'is_scratched':         True,
            'horse_url':            'https://www.punters.com.au/horses/All-Australian-Gal_323668/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Brad-Rawiller_179/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/Luke-Oliver_734/',
            'weight':               57.0,
            'barrier':              6,
            'scraper_version':      punters_client.__version__
        },
        {
            'number':               9,
            'is_scratched':         True,
            'horse_url':            'https://www.punters.com.au/horses/Rose-Dancer_315350/',
            'horse_has_blinkers':   False,
            'jockey_url':           'https://www.punters.com.au/jockeys/Ryan-Maloney_1490/',
            'jockey_is_apprentice': False,
            'jockey_claiming':      0.0,
            'trainer_url':          'https://www.punters.com.au/trainers/Luke-Oliver_734/',
            'weight':               56.5,
            'barrier':              8,
            'scraper_version':      punters_client.__version__
        }
    ]


@pytest.fixture(scope='module')
def scraped_runners(race, scraper):

    return scraper.scrape_runners(race)


def test_expected_runners(expected_runners, scraped_runners):
    """The scrape_runners method should return a list of dictionaries containing all expected values"""

    check_expected_items(expected_runners, scraped_runners, ['number'])


def test_unexpected_runners(expected_runners, scraped_runners):
    """The scrape_runners method should return a list of dictionaries that does not contain any unexpected values"""

    check_unexpected_items(expected_runners, scraped_runners, ['number'])



def check_expected_items(expected_items, scraped_items, keys):
    """Assert that all expected items exist in the scraped items collection"""

    for expected_item in expected_items:
        scraped_item = find(expected_item, scraped_items, keys)

        for key in expected_item:
            assert scraped_item[key] == expected_item[key]
        for key in scraped_item:
            assert expected_item[key] == scraped_item[key]


def check_unexpected_items(expected_items, scraped_items, keys):
    """Assert that all scraped items exist in the expected items collection"""

    for scraped_item in scraped_items:
        assert find(scraped_item, expected_items, keys) is not None


def find(item, collection, keys):
    """Find the item in the specified collection whose keys/values match those in the specified item"""

    for other_item in collection:
        is_match = True
        for key in keys:
            if other_item[key] != item[key]:
                is_match = False
                break
        if is_match:
            return other_item



from datetime import datetime

import punters_client
import pytest

from common import check_expected_items, check_unexpected_items


@pytest.fixture(scope='module')
def horse(source_timezone):

    return {
        'url':              'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
        'name':             'Tycoon Tony',
        'colour':           'bay',
        'sex':              'gelding',
        'sire':             'Written Tycoon',
        'dam':              'Real Ruby',
        'country':          'Australia',
        'foaled':           source_timezone.localize(datetime(2011, 10, 1)),
        'scraper_version':  punters_client.__version__
    }


@pytest.fixture(scope='module')
def expected_performances(source_timezone):

    return [
        {
            'track':            'Wangaratta',
            'date':             source_timezone.localize(datetime(2016, 4, 21)),
            'distance':         1170,
            'track_condition':  'GOOD 4',
            'prize_money':      500.00,
            'prize_pool':       20000.00,
            'barrier':          16,
            'winning_time':     68.66,
            'starting_price':   6.50,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Dylan-Dunn_7917/',
            'weight':           60.5,
            'carried':          60.5,
            'lengths':          4.15,
            'result':           10,
            'starters':         13,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Echuca',
            'date':             source_timezone.localize(datetime(2016, 4, 10)),
            'distance':         1209,
            'track_condition':  'GOOD 4',
            'prize_money':      1600.00,
            'prize_pool':       20000.00,
            'barrier':          15,
            'winning_time':     73.20,
            'starting_price':   6.50,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Mark-Pegus_980/',
            'weight':           60.5,
            'carried':          60.5,
            'lengths':          0.30,
            'result':           3,
            'starters':         12,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Yarra Valley',
            'date':             source_timezone.localize(datetime(2016, 3, 20)),
            'distance':         1200,
            'track_condition':  'HEAVY 9',
            'prize_money':      2000.00,
            'prize_pool':       25000.00,
            'barrier':          4,
            'winning_time':     76.55,
            'starting_price':   2.10,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Ben-Allen_9146/',
            'weight':           60.5,
            'carried':          59.0,
            'lengths':          2.95,
            'result':           3,
            'starters':         4,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Sandown Hillside',
            'date':             source_timezone.localize(datetime(2016, 2, 17)),
            'distance':         1300,
            'track_condition':  'SOFT 5',
            'prize_money':      700.00,
            'prize_pool':       35000.00,
            'barrier':          4,
            'winning_time':     79.18,
            'starting_price':   9.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Harry-Coffey_3109/',
            'weight':           61.0,
            'carried':          59.5,
            'lengths':          4.35,
            'result':           6,
            'starters':         11,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Kilmore',
            'date':             source_timezone.localize(datetime(2016, 2, 1)),
            'distance':         1100,
            'track_condition':  'GOOD 4',
            'prize_money':      3400.00,
            'prize_pool':       20000.00,
            'barrier':          2,
            'winning_time':     63.60,
            'starting_price':   4.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Ben-E-Thompson_2143/',
            'weight':           61.0,
            'carried':          59.5,
            'lengths':          0.10,
            'result':           2,
            'starters':         8,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Wangaratta',
            'date':             source_timezone.localize(datetime(2016, 1, 21)),
            'distance':         1300,
            'track_condition':  'GOOD 4',
            'prize_money':      500.00,
            'prize_pool':       20000.00,
            'barrier':          12,
            'winning_time':     77.55,
            'starting_price':   8.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Mitchell-Aitken_9402/',
            'weight':           61.0,
            'carried':          58.0,
            'lengths':          5.25,
            'result':           9,
            'starters':         12,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Wangaratta',
            'date':             source_timezone.localize(datetime(2016, 1, 4)),
            'distance':         1170,
            'track_condition':  'SOFT 5',
            'prize_money':      11000.00,
            'prize_pool':       20000.00,
            'barrier':          5,
            'winning_time':     68.51,
            'starting_price':   10.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Ben-E-Thompson_2143/',
            'weight':           60.0,
            'carried':          58.5,
            'lengths':          0.00,
            'result':           1,
            'starters':         14,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Echuca',
            'date':             source_timezone.localize(datetime(2015, 9, 25)),
            'distance':         1100,
            'track_condition':  'GOOD 4',
            'prize_money':      600.00,
            'prize_pool':       20000.00,
            'barrier':          1,
            'winning_time':     64.42,
            'starting_price':   7.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Jason-Collins_2265/',
            'weight':           59.0,
            'carried':          57.0,
            'lengths':          2.80,
            'result':           5,
            'starters':         11,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Echuca',
            'date':             source_timezone.localize(datetime(2015, 9, 5)),
            'distance':         1200,
            'track_condition':  'GOOD 4',
            'prize_money':      13000.00,
            'prize_pool':       20000.00,
            'barrier':          12,
            'winning_time':     72.04,
            'starting_price':   19.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Jason-Collins_2265/',
            'weight':           58.5,
            'carried':          56.5,
            'lengths':          0.00,
            'result':           1,
            'starters':         13,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Echuca',
            'date':             source_timezone.localize(datetime(2015, 8, 17)),
            'distance':         1400,
            'track_condition':  'GOOD 4',
            'prize_money':      900.00,
            'prize_pool':       20000.00,
            'barrier':          6,
            'winning_time':     85.74,
            'starting_price':   5.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Jason-Benbow_690/',
            'weight':           58.5,
            'carried':          58.5,
            'lengths':          5.25,
            'result':           4,
            'starters':         14,
            'scraper_version':  punters_client.__version__
        },
        {
            'track':            'Wodonga',
            'date':             source_timezone.localize(datetime(2015, 7, 29)),
            'distance':         1100,
            'track_condition':  'HEAVY 9',
            'prize_money':      1200.00,
            'prize_pool':       15000.00,
            'barrier':          15,
            'winning_time':     67.96,
            'starting_price':   21.00,
            'horse_url':        'https://www.punters.com.au/horses/Tycoon-Tony_395471/',
            'jockey_url':       'https://www.punters.com.au/jockeys/Jason-Benbow_690/',
            'weight':           58.5,
            'carried':          58.5,
            'lengths':          4.50,
            'result':           3,
            'starters':         11,
            'scraper_version':  punters_client.__version__
        }
    ]


@pytest.fixture(scope='module')
def scraped_performances(horse, scraper):

    return scraper.scrape_performances(horse)


def test_expected_performances(expected_performances, scraped_performances):
    """The scrape_performances method should return a list of dictionaries containing all expected values"""

    check_expected_items(expected_performances, scraped_performances, ['date', 'horse_url', 'track'])


def test_unexpected_performances(expected_performances, scraped_performances):
    """The scrape_performances method should return a list of dictionaries that does not contain any unexpected values"""

    check_unexpected_items(expected_performances, scraped_performances, ['date', 'horse_url', 'track'])


def test_mangled_starting_price_42(scraper, source_timezone):
    """The scrape_performances method should handle mangled starting prices (see issue #42)"""

    horse = {
        'url':  'https://www.punters.com.au/horses/Husna_219457/'
    }

    performances = scraper.scrape_performances(horse)

    found_performance = False
    for performance in performances:
        if performance['date'] == source_timezone.localize(datetime(2013, 4, 27)) and performance['track'] == 'Hobart':
            found_performance = True
            assert performance['starting_price'] == 22.70
            break

    assert found_performance


def test_mangled_starting_price_44(scraper, source_timezone):
    """The scrape_performances method should handle mangled starting prices (see issue #44)"""

    horse = {
        'url':  'https://www.punters.com.au/horses/Glove-Man_223429/'
    }

    performances = scraper.scrape_performances(horse)

    found_performance = False
    for performance in performances:
        if performance['date'] == source_timezone.localize(datetime(2013, 3, 13)) and performance['track'] == 'Matamata':
            found_performance = True
            assert performance['starting_price'] == 2.80
            break

    assert found_performance



import pytest


@pytest.fixture(scope='module', params=['test/path', '/test/path'])
def url(request):

    return request.param


@pytest.fixture(scope='module', params=['http://www.example.com', 'http://www.example.com/'])
def url_root(request):

    return request.param


def test_slashes(scraper, url, url_root):
    """The fix_url method should always return the correct number of slashes between url_root and url"""

    assert scraper.fix_url(url, url_root) == 'http://www.example.com/test/path'



import pytest


def test_4xx(scraper):
    """The get_html method should return None when receiving a HTTP 4xx status code"""

    assert scraper.get_html('http://httpstat.us/404') is None


def test_5xx(scraper):
    """The get_html method should raise an exception when receiving a HTTP 5xx status code"""

    with pytest.raises(BaseException):
        scraper.get_html('http://httpstat.us/500')



from datetime import datetime

import punters_client
import pytest


@pytest.fixture(scope='module')
def expected_values(runner, source_timezone):

    return {
        'url':              runner['horse_url'],
        'name':             'Tycoon Tony',
        'colour':           'bay',
        'sex':              'gelding',
        'sire':             'Written Tycoon',
        'dam':              'Real Ruby',
        'country':          'Australia',
        'foaled':           source_timezone.localize(datetime(2011, 10, 1)),
        'scraper_version':  punters_client.__version__
    }


@pytest.fixture(scope='module')
def scraped_values(runner, scraper):

    return scraper.scrape_horse(runner)


def test_expected_values(expected_values, scraped_values):
    """The scrape_horse method should return a dictionary containing all expected values"""

    assert scraped_values == expected_values
